//====================================================================
//Project Lynxmotion Phoenix
//Description: Phoenix, control file.
//The control input subroutine for the phoenix software is placed in this file.
//Can be used with V2.0 and above
//Configuration version: V1.0
//Date: 03-10-2017
//Programmer: Frank Friedhof port to PS3 USB/BT, Jeroen Janssen (aka Xan)
//             Kurt Eckhardt (aka KurtE) - converted to c ported to Arduino...
//
//Hardware setup: PS3 version
// 
//NEW IN V1.0
//- First Release
//
//Walk method 1:
//- Left Stick    Walk/Strafe
//- Right Stick   Rotate
//
//Walk method 2:
//- Left Stick    Disable
//- Right Stick   Walk/Rotate
//
//
//PS3 CONTROLS:
//[Common Controls]
//- StartTurn     on/off the bot
//- L1            Toggle Shift mode
//- L2            Toggle Rotate mode
//- Circle        Toggle Single leg mode
//- Square        Toggle Balance mode
//- Triangle      Move body to 35 mm from the ground (walk pos) 
//and back to the ground
//- D-Pad up      Body up 10 mm
//- D-Pad down    Body down 10 mm
//- D-Pad left    decrease speed with 50mS
//- D-Pad right   increase speed with 50mS
//
// Optional: L3 button down, Left stick can adjust leg positions...
// or if OPT_SINGLELEG is not defined may try using Circle

//
//
//[Walk Controls]
//- select        Switch gaits
//- Left Stick    (Walk mode 1) Walk/Strafe
//                (Walk mode 2) Disable
//- Right Stick   (Walk mode 1) Rotate, 
//                (Walk mode 2) Walk/Rotate
//- R1            Toggle Double gait travel speed
//- R2            Toggle Double gait travel length
//
//[Shift Controls]
//- Left Stick    Shift body X/Z
//- Right Stick   Shift body Y and rotate body Y
//
//[Rotate Controls]
//- Left Stick    Rotate body X/Z
//- Right Stick   Rotate body Y
//
//[Single leg Controls]
//- select        Switch legs
//- Left Stick    Move Leg X/Z (relative)
//- Right Stick   Move Leg Y (absolute)
//- R2            Hold/release leg position
//
//[GP Player Controls]
//- select        Switch Sequences
//- R2            Start Sequence
//
//====================================================================
// [Include files]
#if ARDUINO>99
#include <Arduino.h> // Arduino 1.0
#else
#include <Wprogram.h> // Arduino 0022
#endif

#include "Hex_Globals.h"

#ifndef MAX_BODY_Y
#define MAX_BODY_Y 120
#endif


#ifdef USEPS3

#ifdef USE_PS3BT
#include <PS3BT.h>
#include <usbhub.h>
#else
#include <PS3USB.h>
#endif

//[CONSTANTS]
#define WALKMODE          0
#define TRANSLATEMODE     1
#define ROTATEMODE        2
#define SINGLELEGMODE     3
#define GPPLAYERMODE      4


#define cTravelDeadZone 4      //The deadzone for the analog input from the remote


//=============================================================================
// Global - Local to this file only...
//=============================================================================
USB Usb;
#ifdef USE_PS3BT
BTD Btd(&Usb); // You have to create the Bluetooth Dongle instance like so
PS3BT ps3x(&Btd, 0x00, 0x11, 0x22, 0x98, 0x76, 0x54); // This will also store the bluetooth address - this can be obtained from the dongle when running the sketch
#else
PS3USB ps3x(&Usb); // This will just create the instance
#endif

// Satisfy the IDE, which needs to see the include statment in the ino too.
#ifdef dobogusinclude
#include <spi4teensy3.h>
#include <SPI.h>
#endif


static unsigned    g_BodyYOffset; 
static char        g_BodyYSift;
static byte        ControlMode;
static bool        DoubleHeightOn;
static bool        DoubleTravelOn;
static bool        WalkMethod;

// some external or forward function references.
extern void MSound(uint8_t _pin, byte cNotes, ...);
extern int DBGPrintf(const char *format, ...);

//==============================================================================
// This is The function that is called by the Main program to initialize
//the input controller, which in this case is the PS2 controller
//process any commands.
//==============================================================================

void InitController(void)
{
  int error;

  //error = ps3x.config_gamepad(57, 55, 56, 54);  // Setup gamepad (clock, command, attention, data) pins
  //error = ps3x.config_gamepad(PS2_CLK, PS2_CMD, PS2_SEL, PS2_DAT);  // Setup gamepad (clock, command, attention, data) pins
  if (Usb.Init() == -1) {
    DBGSerial.print(F("\r\nOSC did not start"));
    while (1); //halt
  }
  DBGSerial.println(F("\r\nPS3 USB/Bluetooth Library Started"));

  g_BodyYOffset = 0;
  g_BodyYSift = 0;
}

//==============================================================================
// This function is called by the main code to tell us when it is about to
// do a lot of bit-bang outputs and it would like us to minimize any interrupts
// that we do while it is active...
//==============================================================================
void AllowControllerInterrupts(boolean fAllow)
{
  // We don't need to do anything...
}

//==============================================================================
// This is The main code to input function to read inputs from the PS3 and then
// process any commands.
//==============================================================================
void ControlInput(void)
{
  // Then try to receive a packet of information from the PS2.
  // Then try to receive a packet of information from the PS2.
  Usb.Task();   // ps2x.read_gamepad()

    // Wish the library had a valid way to verify that the read_gamepad succeeded...
    //  Will hack for now
  if (ps3x.PS3Connected || ps3x.PS3NavigationConnected) {
    // In an analog mode so should be OK...

    if (ps3x.getButtonClick(START)) {// OK lets press start button
      DBGPrintf("Start button Pressed\r");
      if (g_fHexOn) {
        //Turn off
        BodyPosX = 0;
        BodyPosY = 0;
        BodyPosZ = 0;
        BodyRotX1 = 0;
        BodyRotY1 = 0;
        BodyRotZ1 = 0;
        TravelLengthX = 0;
        TravelLengthZ = 0;
        TravelRotationY = 0;
        g_BodyYOffset = 0;
        g_BodyYSift = 0;
        SelectedLeg = 255;
        g_fHexOn = 0;
      } else {
        //Turn on
        g_fHexOn = 1;
      }
    }

    if (g_fHexOn) {
      // [SWITCH MODES]

      //Translate mode
      if (ps3x.getButtonClick(L1)) {// L1 Button Test
        MSound( SOUND_PIN, 1, 50, 2000);  
        if (ControlMode != TRANSLATEMODE ) {
          ControlMode = TRANSLATEMODE;
          DBGSerial.println("Translate mode");
        }
        else {
          if (SelectedLeg == 255) {
            ControlMode = WALKMODE;
            DBGSerial.println("Walk mode");
          }
          else
          {
            ControlMode = SINGLELEGMODE;
            DBGSerial.println("Single leg mode");
          }
        }
      }

      //Rotate mode
      if (ps3x.getButtonClick(L2)) {    // L2 Button Test
        MSound( SOUND_PIN, 1, 50, 2000);
        if (ControlMode != ROTATEMODE) {
          ControlMode = ROTATEMODE;
          DBGSerial.println("Rotate mode");
        }
        else {
          if (SelectedLeg == 255) {
            ControlMode = WALKMODE;
            DBGSerial.println("Walk mode");
          }
          else
          {
            ControlMode = SINGLELEGMODE;
            DBGSerial.println("Single leg mode");
          }
        }
      }

      //Single leg mode fNO
      if (ps3x.getButtonClick(CIRCLE)) {// O - Circle Button Test
        if (abs(TravelLengthX)<cTravelDeadZone && abs(TravelLengthZ)<cTravelDeadZone 
          && abs(TravelRotationY*2)<cTravelDeadZone )   {
          if (ControlMode != SINGLELEGMODE) {
            ControlMode = SINGLELEGMODE;
            DBGSerial.println("Single leg mode");
            if (SelectedLeg == 255)  //Select leg if none is selected
              SelectedLeg=cRF; //Startleg
          } 
          else {
            ControlMode = WALKMODE;
            DBGSerial.println("Walk mode");
            SelectedLeg=255;
          }
        }
      }      

      // GP Player Mode X
      if (ps3x.getButtonClick(CROSS)) { // X - Cross Button Test
        MSound(SOUND_PIN, 1, 50, 2000);  
        if (ControlMode != GPPLAYERMODE) {
          ControlMode = GPPLAYERMODE;
          GPSeq=0;
          DBGSerial.println("GP player mode");
        } 
        else {
          ControlMode = WALKMODE;
          DBGSerial.println("Walk mode");
        }
      }

      //[Common functions]
      //Switch Balance mode on/off 
      if (ps3x.getButtonClick(SQUARE)) { // Square Button Test
        BalanceMode = !BalanceMode;
        if (BalanceMode) {
          MSound(SOUND_PIN, 1, 250, 1500);
          DBGSerial.println("Balance mode ON");
        } 
        else {
          MSound(SOUND_PIN, 2, 100, 2000, 50, 4000);
          DBGSerial.println("Balance mode OFF");
        }
      }

      //Stand up, sit down  
      if (ps3x.getButtonClick(TRIANGLE)) { // Triangle - Button Test
        if (g_BodyYOffset>0) 
          g_BodyYOffset = 0;
        else
          g_BodyYOffset = 35;
        DBGSerial.print("BodyYOffset : ");
        DBGSerial.println(g_BodyYOffset);
      }

      if (ps3x.getButtonClick(UP)) {// D-Up - Button Test
        g_BodyYOffset += 10;
        g_BodyYOffset = min(g_BodyYOffset, MAX_BODY_Y);

        DBGSerial.print("BodyYOffset : ");
        DBGSerial.println(g_BodyYOffset);
      }

      if (ps3x.getButtonClick(DOWN) && g_BodyYOffset) {// D-Down - Button Test
        if (g_BodyYOffset > 10)
          g_BodyYOffset -= 10;
        else
          g_BodyYOffset = 0;      // constrain don't go less than zero.

        DBGSerial.print("BodyYOffset : ");
        DBGSerial.println(g_BodyYOffset);
      }

      if (ps3x.getButtonClick(RIGHT)) { // D-Right - Button Test
        if (SpeedControl>0) {
          SpeedControl = SpeedControl - 50;
          MSound(SOUND_PIN, 1, 50, 2000);  
        }
        DBGSerial.print("SpeedControl : ");
        DBGSerial.println(SpeedControl);
      }

      if (ps3x.getButtonClick(LEFT)) { // D-Left - Button Test
        if (SpeedControl<2000 ) {
          SpeedControl = SpeedControl + 50;
          MSound(SOUND_PIN, 1, 50, 2000); 
        }
        DBGSerial.print("SpeedControl : ");
        DBGSerial.println(SpeedControl);
      }
      
      //[Walk functions]
      if (ControlMode == WALKMODE) {
        //Switch gates
        if (ps3x.getButtonPress(SELECT)            // Select Button Test
              && abs(TravelLengthX)<cTravelDeadZone //No movement
              && abs(TravelLengthZ)<cTravelDeadZone 
              && abs(TravelRotationY*2)<cTravelDeadZone  ) {
          if (GaitType < 7) {                 // Make sure we did not exceed number of gaits...
            MSound(SOUND_PIN, 1, 50, 2000); 
            GaitType = GaitType+1;                    // Go to the next gait...
          } 
          else {
            MSound(SOUND_PIN, 2, 50, 2000, 50, 2250); 
            GaitType = 0;
          }
          GaitSelect();
        }

        //Double leg lift height
        if (ps3x.getButtonClick(R1)) { // R1 Button Test
          MSound(SOUND_PIN, 1, 50, 2000); 
          DoubleHeightOn = !DoubleHeightOn;
          if (DoubleHeightOn)
            LegLiftHeight = 80;
          else
            LegLiftHeight = 50;
          DBGSerial.print("Gait level ");
          DBGSerial.println(LegLiftHeight);
        }

        //Double Travel Length
        if (ps3x.getButtonPress(R2)) {// R2 Button Test
          MSound(SOUND_PIN, 1, 50, 2000); 
          DoubleTravelOn = !DoubleTravelOn;
          DBGSerial.print("Gait speed ");
          DBGSerial.println(DoubleTravelOn?"normal":"double");
        }

        // Switch between Walk method 1 && Walk method 2
        if (ps3x.getButtonPress(R3)) { // R3 Button Test
          MSound(SOUND_PIN, 50, 2000); 
          WalkMethod = !WalkMethod;
          DBGSerial.print("Walk method ");
          DBGSerial.println(WalkMethod?"1":"2");
        }

        //Walking
        if (WalkMethod)  //(Walk Methode) 
          TravelLengthZ = (ps3x.getAnalogHat(RightHatY)-128); //Right Stick Up/Down  

        else {
          TravelLengthX = -(ps3x.getAnalogHat(LeftHatX) - 128);
          TravelLengthZ = (ps3x.getAnalogHat(LeftHatY) - 128);
        }

        if (!DoubleTravelOn) {  //(Double travel length)
          TravelLengthX = TravelLengthX/2;
          TravelLengthZ = TravelLengthZ/2;
        }

        TravelRotationY = -(ps3x.getAnalogHat(RightHatX) - 128)/4; //Right Stick Left/Right 
      }

      //[Translate functions]
      //g_BodyYShift = 0;
      if (ControlMode == TRANSLATEMODE) {
        BodyPosX = (ps3x.getAnalogHat(LeftHatX) - 128)/2;
        BodyPosZ = -(ps3x.getAnalogHat(LeftHatY) - 128)/3;
        BodyRotY1 = (ps3x.getAnalogHat(RightHatX) - 128)*2;
        g_BodyYSift = (-(ps3x.getAnalogHat(RightHatY) - 128)/2);
      }

      //[Rotate functions]
      if (ControlMode == ROTATEMODE) {
        BodyRotX1 = (ps3x.getAnalogHat(LeftHatY) - 128);
        BodyRotY1 = (ps3x.getAnalogHat(RightHatX) - 128)*2;
        BodyRotZ1 = (ps3x.getAnalogHat(LeftHatX) - 128);
        g_BodyYSift = (-(ps3x.getAnalogHat(RightHatY) - 128)/2);
      }

      //[Single leg functions]
      if (ControlMode == SINGLELEGMODE) {
        //Switch leg for single leg control
        if (ps3x.getButtonClick(SELECT)) { // Select Button Test
          MSound(SOUND_PIN, 1, 50, 2000); 
          if (SelectedLeg<5)
            SelectedLeg = SelectedLeg+1;
          else
            SelectedLeg=0;
        }

        SLLegX= (ps3x.getAnalogHat(LeftHatX) - 128)/2; //Left Stick Right/Left
        SLLegY= (ps3x.getAnalogHat(RightHatY) - 128)/2; //Right Stick Up/Down
        SLLegZ = (ps3x.getAnalogHat(LeftHatY) - 128)/2; //Left Stick Up/Down

        // Hold single leg in place
        if (ps3x.getButtonPress(R2)) { // R2 Button Test
          MSound(SOUND_PIN, 1, 50, 2000);  
          fSLHold = !fSLHold;
        }
      }

      //[GPPlayer functions]
      if (ControlMode == GPPLAYERMODE) {

          //Switch between sequences
          if (ps3x.getButtonClick(SELECT)) { // Select Button Test
              if (GPStart==0 ) {
                  if (GPSeq < 5) {  //Max sequence
                      MSound (SOUND_PIN, 1, 50, 1500);  //sound SOUND_PIN, [50\3000]
                      GPSeq = GPSeq+1;
                  } else {
                      MSound (SOUND_PIN, 2, 50, 2000, 50, 2250);//Sound SOUND_PIN,[50\4000, 50\4500]
                      GPSeq=0;
                  }
              }
          }
          //Start Sequence
          if (ps3x.getButtonClick(R2))// R2 Button Test
              GPStart=1;
      }

      //Calculate walking time delay
      InputTimeDelay = 128 - max(max(abs(ps3x.getAnalogHat(LeftHatX)  - 128), abs(ps3x.getAnalogHat(LeftHatY)  - 128)), abs(ps3x.getAnalogHat(RightHatX) - 128));
    }

    //Calculate BodyPosY
    BodyPosY = max(g_BodyYOffset + g_BodyYSift,  0);
  } 

}

#endif //USEPS3

